package pract09_dyv_2_mayores;

/**
 * Algoritmos para la bsqueda de los dos mayores elementos de un array
 * - DyV
 * - Bsqueda directa
 * 
 * @author mario
 *
 */
public class Busca2mayores {
	
	/**
	 * Busca los dos mayores elementos del array utilizando un algorimo DyV.
	 * Llama a busca2mayoresRec.
	 * Asume que el array tiene un nmero par de elementos.
	 * @param a array en el que buscar
	 * @return un array con los dos mayores elementos
	 */
	public static int[] busca2mayores(int[] a) {
		return busca2mayoresRec(a, 0, a.length-1);
	}
	
	/**
	 * Busca los dos mayores elementos del trozo del array comprendido entre 
	 * los ndices ini y fin utilizando un algorimo DyV.
	 * Asume que el array tiene un nmero par de elementos
	 * @param a array en el que buscar
	 * @param ini comienzo del trozo en el que buscar los dos mximos
	 * @param fin final del trozo en el que buscar los dos mximos
	 * @return un array con los dos mayores elementos
	 */
	private static int[] busca2mayoresRec(int[] a, int ini, int fin) {
		// caso directo: trozo en el que buscar es de tamao 2
		if (fin - ini + 1 == 2)
			if (a[ini] > a[fin]) {
				int[] ret = {a[ini], a[fin]};
				return ret;
			} else {
				int[] ret = {a[fin], a[ini]};
				return ret;
			}
		
		// llamadas recursivas
		int m = (ini + fin) / 2;
		int[] r1 = busca2mayoresRec(a, ini, m);
		int[] r2 = busca2mayoresRec(a, m+1, fin);
		
		// recombinacin de las soluciones
		if (r1[0] > r2[0]) {
			if (r2[0] > r1[1])
				r1[1] = r2[0];
			return r1;
		} else {
			if (r1[0] > r2[1])
				r2[1] = r1[0];
			return r2;
		}			
	}
	
	/**
	 * Busca los dos mayores elementos del array utilizando una bsqueda lineal
	 * sobre todos los elementos del array.
	 * 
	 * @param a array en el que buscar
	 * @return un array con los dos mayores elementos
	 */
	public static int[] busca2mayoresLineal(int[] a) {
		int[] ret = {Integer.MIN_VALUE, Integer.MIN_VALUE};
		
		for(int i:a)
			if (i>ret[0]) {
				ret[1] = ret[0];
				ret[0] = i;
			} else if (i>ret[1])
				ret[1] = i;
		
		return ret;
	}

	

}
